@@ -11,29 +11,254 @@
 
 #define ACCESS_CMP(x, y) ((st64)((ut64)(x) - (ut64)((RzAnalysisVarAccess *)y)->offset))
 
-static bool storage_equals(RzAnalysisVarStorage *a, RzAnalysisVarStorage *b) {
+/**
+ * \brief Compare two RzAnalysisVarAccess objects.
+ * \param a RzAnalysisVarStorage object.
+ * \param b RzAnalysisVarStorage object.
+ * \return 0 if equal, negative if a < b, positive if a > b.
+ */
+RZ_API int rz_analysis_var_storage_cmp(
+	RZ_NONNULL const RzAnalysisVarStorage *a,
+	RZ_NONNULL const RzAnalysisVarStorage *b) {
+	rz_return_val_if_fail(a && b, 0);
+	if (a->type != b->type) {
+		return -(int)a->type + (int)b->type;
+	}
 	switch (a->type) {
 	case RZ_ANALYSIS_VAR_STORAGE_REG:
 		// Hint: this strcmp could be optimized to pointer comparison if we add the requirement that a->reg and b->reg
 		// must come from the RzAnalysis.contpool.
-		return b->type == RZ_ANALYSIS_VAR_STORAGE_REG && !strcmp(a->reg, b->reg);
+		return strcmp(a->reg, b->reg);
 	case RZ_ANALYSIS_VAR_STORAGE_STACK:
-		return b->type == RZ_ANALYSIS_VAR_STORAGE_STACK && a->stack_off == b->stack_off;
+		return a->stack_off - b->stack_off;
+	case RZ_ANALYSIS_VAR_STORAGE_COMPOSITE: {
+		RzAnalysisVarStoragePiece *ap = NULL;
+		ut32 i = 0;
+		rz_vector_enumerate(a->composite, ap, i) {
+			RzAnalysisVarStoragePiece *bp = rz_vector_index_ptr(b->composite, i);
+			int xcmp = ap->offset_in_bits - bp->offset_in_bits;
+			if (xcmp != 0) {
+				return xcmp;
+			}
+			xcmp = ap->size_in_bits - bp->size_in_bits;
+			if (xcmp != 0) {
+				return xcmp;
+			}
+			xcmp = rz_analysis_var_storage_cmp(ap->storage, bp->storage);
+			if (xcmp != 0) {
+				return xcmp;
+			}
+		}
+		return 0;
+	}
+	case RZ_ANALYSIS_VAR_STORAGE_EVAL_PENDING:
+		return -1;
 	default:
 		rz_warn_if_reached();
-		return false;
+		return -1;
 	}
 }
 
+/**
+ * \brief Check if two RzAnalysisVarStorage objects are equal.
+ * \see rz_analysis_var_storage_cmp
+ * \param a RzAnalysisVarStorage object.
+ * \param b RzAnalysisVarStorage object.
+ * \return true if equal, false otherwise.
+ */
+RZ_API bool rz_analysis_var_storage_equals(
+	RZ_NONNULL const RzAnalysisVarStorage *a,
+	RZ_NONNULL const RzAnalysisVarStorage *b) {
+	rz_return_val_if_fail(a && b, 0);
+	return rz_analysis_var_storage_cmp(a, b) == 0;
+}
+
+static const char *var_storage_strings[] = {
+	[RZ_ANALYSIS_VAR_STORAGE_REG] = "reg",
+	[RZ_ANALYSIS_VAR_STORAGE_STACK] = "stack",
+	[RZ_ANALYSIS_VAR_STORAGE_COMPOSITE] = "composite",
+	[RZ_ANALYSIS_VAR_STORAGE_EVAL_PENDING] = "eval_pending",
+};
+
+RZ_API const char *rz_analysis_var_storage_type_to_string(RzAnalysisVarStorageType type) {
+	if (type > RZ_ANALYSIS_VAR_STORAGE_EVAL_PENDING || type < 0) {
+		return NULL;
+	}
+	return var_storage_strings[type];
+}
+
+RZ_API bool rz_analysis_var_storage_type_from_string(
+	RZ_NONNULL const char *type_str,
+	RZ_NONNULL RZ_BORROW RZ_OUT RzAnalysisVarStorageType *type) {
+	rz_return_val_if_fail(type_str && type, false);
+	for (int i = 0; i <= RZ_ANALYSIS_VAR_STORAGE_EVAL_PENDING; ++i) {
+		if (RZ_STR_EQ(type_str, var_storage_strings[i])) {
+			*type = i;
+			return true;
+		}
+	}
+	return false;
+}
+
+static void strbuf_append_sign_hex(RzStrBuf *sb, st64 x) {
+	const char sign = x >= 0 ? '+' : '-';
+	rz_strbuf_appendf(sb, " %c 0x%" PFMT64x, sign, RZ_ABS(x));
+}
+
+RZ_API void rz_analysis_var_storage_dump(
+	RZ_NONNULL RZ_BORROW RzAnalysis *a,
+	RZ_NONNULL RZ_BORROW RZ_OUT RzStrBuf *sb,
+	RZ_NULLABLE RZ_BORROW const RzAnalysisVar *var,
+	RZ_NONNULL RZ_BORROW const RzAnalysisVarStorage *storage) {
+	rz_return_if_fail(a && sb && storage);
+	switch (storage->type) {
+	case RZ_ANALYSIS_VAR_STORAGE_REG: {
+		rz_strbuf_append(sb, storage->reg);
+		break;
+	}
+	case RZ_ANALYSIS_VAR_STORAGE_STACK: {
+		rz_strbuf_append(sb, var_storage_strings[storage->type]);
+		strbuf_append_sign_hex(sb, storage->stack_off);
+		break;
+	}
+	case RZ_ANALYSIS_VAR_STORAGE_COMPOSITE: {
+		rz_strbuf_append(sb, "COMPOSITE");
+		break;
+	}
+	case RZ_ANALYSIS_VAR_STORAGE_EVAL_PENDING:
+		/// Omit storage information
+		if (var && var->origin.kind == RZ_ANALYSIS_VAR_ORIGIN_DWARF &&
+			var->origin.dw_var && var->origin.dw_var->location &&
+			var->origin.dw_var->location->kind == RzBinDwarfLocationKind_LOCLIST) {
+			rz_strbuf_append(sb, "LOCLIST");
+		} else {
+			rz_strbuf_append(sb, "...");
+		}
+		break;
+	default:
+		rz_warn_if_reached();
+		break;
+	}
+}
+
+RZ_API RZ_OWN char *rz_analysis_var_storage_to_string(
+	RZ_NONNULL RZ_BORROW RzAnalysis *a,
+	RZ_NULLABLE RZ_BORROW const RzAnalysisVar *var,
+	RZ_NONNULL RZ_BORROW const RzAnalysisVarStorage *storage) {
+	rz_return_val_if_fail(a && storage, NULL);
+	RzStrBuf *sb = rz_strbuf_new(NULL);
+	rz_analysis_var_storage_dump(a, sb, var, storage);
+	return rz_strbuf_drain(sb);
+}
+
+static void composite_dump_pj(
+	RZ_NONNULL RZ_BORROW RZ_OUT PJ *pj,
+	RZ_NONNULL RZ_BORROW const RzAnalysisVar *var,
+	RZ_NONNULL RZ_BORROW const RzVector /*<RzAnalysisVarStoragePiece>*/ *composite) {
+	pj_a(pj);
+	RzAnalysisVarStoragePiece *piece = NULL;
+	rz_vector_foreach(composite, piece) {
+		pj_o(pj);
+		pj_kn(pj, "offset_in_bits", piece->offset_in_bits);
+		pj_kn(pj, "size_in_bits", piece->size_in_bits);
+		rz_analysis_var_storage_dump_pj(pj, var, piece->storage);
+		pj_end(pj);
+	}
+	pj_end(pj);
+}
+
+RZ_API void rz_analysis_var_storage_dump_pj(
+	RZ_NONNULL RZ_BORROW RZ_OUT PJ *pj,
+	RZ_NONNULL RZ_BORROW const RzAnalysisVar *var,
+	RZ_NONNULL RZ_BORROW const RzAnalysisVarStorage *storage) {
+	rz_return_if_fail(pj && var && storage);
+	const char *type = rz_analysis_var_storage_type_to_string(storage->type);
+	pj_k(pj, "storage");
+	pj_o(pj);
+	pj_ks(pj, "type", rz_str_get_null(type));
+	if (type) {
+		pj_k(pj, type);
+		switch (storage->type) {
+		case RZ_ANALYSIS_VAR_STORAGE_STACK:
+			pj_N(pj, storage->stack_off);
+			break;
+		case RZ_ANALYSIS_VAR_STORAGE_REG:
+			pj_s(pj, storage->reg);
+			break;
+		case RZ_ANALYSIS_VAR_STORAGE_COMPOSITE:
+			composite_dump_pj(pj, var, storage->composite);
+			break;
+		case RZ_ANALYSIS_VAR_STORAGE_EVAL_PENDING:
+			if (var->origin.kind == RZ_ANALYSIS_VAR_ORIGIN_DWARF) {
+				pj_n(pj, var->origin.dw_var->offset);
+			} else {
+				rz_warn_if_reached();
+			}
+			break;
+		default:
+			rz_warn_if_reached();
+			break;
+		}
+	}
+	pj_end(pj);
+}
+
 /**
  * Ensure that the register name in \p stor comes from the const pool
  */
-static void storage_poolify(RzAnalysis *analysis, RzAnalysisVarStorage *stor) {
+RZ_API void rz_analysis_var_storage_poolify(
+	RZ_NONNULL RZ_BORROW RzAnalysis *analysis,
+	RZ_NONNULL RZ_BORROW RZ_OUT RzAnalysisVarStorage *stor) {
+	rz_return_if_fail(analysis && stor);
 	if (stor->type == RZ_ANALYSIS_VAR_STORAGE_REG) {
 		stor->reg = rz_str_constpool_get(&analysis->constpool, stor->reg);
+	} else if (stor->type == RZ_ANALYSIS_VAR_STORAGE_COMPOSITE) {
+		if (!stor->composite) {
+			return;
+		}
+		RzAnalysisVarStoragePiece *piece = NULL;
+		rz_vector_foreach(stor->composite, piece) {
+			rz_analysis_var_storage_poolify(analysis, piece->storage);
+		}
 	}
 }
 
+static void RzVector_RzAnalysisVarStoragePiece_fini(void *e, void *u) {
+	rz_analysis_var_storage_piece_fini(e);
+}
+
+RZ_API void rz_analysis_var_storage_init_composite(RzAnalysisVarStorage *sto) {
+	rz_return_if_fail(sto);
+	sto->type = RZ_ANALYSIS_VAR_STORAGE_COMPOSITE;
+	sto->composite = rz_vector_new(
+		sizeof(RzAnalysisVarStoragePiece), RzVector_RzAnalysisVarStoragePiece_fini, NULL);
+}
+
+RZ_API void rz_analysis_var_storage_piece_fini(RzAnalysisVarStoragePiece *p) {
+	if (!p) {
+		return;
+	}
+	rz_analysis_var_storage_free(p->storage);
+}
+
+RZ_API void rz_analysis_var_storage_fini(RzAnalysisVarStorage *sto) {
+	if (!sto) {
+		return;
+	}
+	if (sto->type == RZ_ANALYSIS_VAR_STORAGE_COMPOSITE) {
+		rz_vector_free(sto->composite);
+		sto->composite = NULL;
+	}
+}
+
+RZ_API void rz_analysis_var_storage_free(RzAnalysisVarStorage *sto) {
+	if (!sto) {
+		return;
+	}
+	rz_analysis_var_storage_fini(sto);
+	free(sto);
+}
+
 static const char *__int_type_from_size(int size) {
 	switch (size) {
 	case 1: return "int8_t";
@@ -52,14 +277,18 @@ static RZ_OWN RzType *var_type_default(RzAnalysis *analysis, int size) {
 	if (!typestr) {
 		typestr = "int32_t";
 	}
-	char *error_msg = NULL;
-	RzType *result = rz_type_parse_string_single(analysis->typedb->parser, typestr, &error_msg);
-	if (!result || error_msg) {
-		RZ_LOG_ERROR("Invalid var type: %s\n%s", typestr, error_msg);
-		free(error_msg);
+	RzType *type = RZ_NEW0(RzType);
+	if (!type) {
 		return NULL;
 	}
-	return result;
+	type->kind = RZ_TYPE_KIND_IDENTIFIER;
+	type->identifier.name = strdup(typestr);
+	if (!type->identifier.name) {
+		free(type);
+		return NULL;
+	}
+	type->identifier.kind = RZ_TYPE_IDENTIFIER_KIND_UNSPECIFIED;
+	return type;
 }
 
 /**
@@ -114,14 +343,23 @@ RZ_API void rz_analysis_var_resolve_overlaps(RzAnalysisVar *var) {
  * \param name a new name to assign to the variable
  * \return the created or updated variable, or NULL if the operation could not be completed
  */
-RZ_API RZ_BORROW RzAnalysisVar *rz_analysis_function_set_var(RzAnalysisFunction *fcn,
-	RZ_NONNULL RzAnalysisVarStorage *stor, RZ_BORROW RZ_NULLABLE const RzType *type, int size, RZ_NONNULL const char *name) {
+RZ_API RZ_BORROW RzAnalysisVar *rz_analysis_function_set_var(
+	RzAnalysisFunction *fcn,
+	RZ_NONNULL RzAnalysisVarStorage *stor,
+	RZ_BORROW RZ_NULLABLE const RzType *type,
+	int size,
+	RZ_NONNULL const char *name) {
 	rz_return_val_if_fail(fcn && name, NULL);
 	RzAnalysisVar *var = rz_analysis_function_get_var_byname(fcn, name);
-	if (var && !storage_equals(&var->storage, stor)) {
-		// var name already exists at a different kind+delta
-		RZ_LOG_WARN("var name %s already exists at a different kind+delta\n", name);
-		return NULL;
+	if (var) {
+		if (!rz_analysis_var_storage_equals(&var->storage, stor)) {
+			// var name already exists at a different kind+delta
+			RZ_LOG_INFO("var name %s already exists at a different kind+delta\n", name);
+			return NULL;
+		} else {
+			// We already have a variable with the same name and storage.
+			return NULL;
+		}
 	}
 	var = rz_analysis_function_get_var_at(fcn, stor);
 	if (!var) {
@@ -134,7 +372,7 @@ RZ_API RZ_BORROW RzAnalysisVar *rz_analysis_function_set_var(RzAnalysisFunction
 
 	var->name = strdup(name);
 	var->storage = *stor;
-	storage_poolify(fcn->analysis, &var->storage);
+	rz_analysis_var_storage_poolify(fcn->analysis, &var->storage);
 	if (type) {
 		if (var->type != type) {
 			rz_type_free(var->type);
@@ -150,6 +388,30 @@ RZ_API RZ_BORROW RzAnalysisVar *rz_analysis_function_set_var(RzAnalysisFunction
 	return var;
 }
 
+static bool var_clean(RzAnalysisVar *var) {
+	void **it;
+	RzPVector *remove_plan = rz_pvector_new((RzPVectorFree)rz_analysis_var_delete);
+	rz_pvector_foreach (&var->fcn->vars, it) {
+		RzAnalysisVar *p = *it;
+		if (p->origin.kind == var->origin.kind) {
+			if (RZ_STR_EQ(p->name, var->name) && rz_analysis_var_storage_equals(&p->storage, &var->storage)) {
+				goto beach;
+			}
+		} else {
+			if (RZ_STR_EQ(p->name, var->name) || rz_analysis_var_storage_equals(&p->storage, &var->storage)) {
+				rz_pvector_push(remove_plan, p);
+			}
+		}
+	}
+	rz_pvector_free(remove_plan);
+	return true;
+beach:
+	rz_analysis_var_free(var);
+	remove_plan->v.free_user = NULL;
+	rz_pvector_free(remove_plan);
+	return false;
+}
+
 /**
  * \brief Add or update a variable \p var to the given function \p fcn.
  *
@@ -161,47 +423,18 @@ RZ_API RZ_BORROW RzAnalysisVar *rz_analysis_function_set_var(RzAnalysisFunction
  *
  * \param fcn the function which the variable will belong to
  * \param var the variable to add or update
- * \param size \p var's type size
  * \return the created or updated variable, or NULL if the operation could not be completed
  */
-RZ_IPI RZ_BORROW RzAnalysisVar *rz_analysis_function_add_var_dwarf(RzAnalysisFunction *fcn, RZ_OWN RzAnalysisVar *var,
-	int size) {
-	rz_return_val_if_fail(fcn && var && var->name, NULL);
-	RzAnalysisVar *old = NULL;
-	void **it;
-	rz_pvector_foreach (&fcn->vars, it) {
-		RzAnalysisVar *p = *it;
-		if (!strcmp(p->name, var->name) || storage_equals(&p->storage, &var->storage)) {
-			old = p;
-		}
+RZ_API RZ_BORROW RzAnalysisVar *rz_analysis_function_add_var(
+	RzAnalysisFunction *fcn, RZ_OWN RzAnalysisVar *var) {
+	rz_return_val_if_fail(fcn && var && var->name && var->type, NULL);
+	if (!var_clean(var)) {
+		return NULL;
 	}
-	if (old) {
-		if (old->kind != RZ_ANALYSIS_VAR_KIND_INVALID) {
-			return NULL;
-		}
-		rz_analysis_var_delete(old);
-	}
-
-	RzAnalysisVar *out = rz_analysis_var_new();
-	out->fcn = fcn;
-	rz_pvector_push(&fcn->vars, out);
-
-	out->name = var->name;
-	out->storage = var->storage;
-	out->kind = var->kind;
-	storage_poolify(fcn->analysis, &out->storage);
-	if (var->type) {
-		if (out->type != var->type) {
-			rz_type_free(out->type);
-			out->type = rz_type_clone(var->type);
-		}
-	} else {
-		if (!out->type) {
-			out->type = var_type_default(fcn->analysis, size);
-		}
-	}
-	rz_analysis_var_resolve_overlaps(out);
-	return out;
+	rz_pvector_push(&fcn->vars, var);
+	rz_analysis_var_storage_poolify(fcn->analysis, &var->storage);
+	rz_analysis_var_resolve_overlaps(var);
+	return var;
 }
 
 /**
@@ -234,10 +467,11 @@ RZ_API void rz_analysis_var_init(RZ_BORROW RzAnalysisVar *var) {
 RZ_API void rz_analysis_var_fini(RZ_OWN RzAnalysisVar *var) {
 	rz_return_if_fail(var);
 	rz_analysis_var_clear_accesses(var);
-	rz_type_free(var->type);
+	RZ_FREE_CUSTOM(var->type, rz_type_free);
 	rz_vector_fini(&var->constraints);
-	free(var->name);
-	free(var->comment);
+	RZ_FREE(var->name);
+	RZ_FREE(var->comment);
+	rz_analysis_var_storage_fini(&var->storage);
 }
 
 RZ_API RZ_OWN RzAnalysisVar *rz_analysis_var_new() {
@@ -349,7 +583,7 @@ RZ_API RZ_BORROW RzAnalysisVar *rz_analysis_function_get_var_at(RzAnalysisFuncti
 	void **it;
 	rz_pvector_foreach (&fcn->vars, it) {
 		RzAnalysisVar *var = *it;
-		if (storage_equals(&var->storage, stor)) {
+		if (rz_analysis_var_storage_equals(&var->storage, stor)) {
 			return var;
 		}
 	}
@@ -553,9 +787,11 @@ RZ_API int rz_analysis_var_get_argnum(RzAnalysisVar *var) {
 	if (!reg) {
 		return -1;
 	}
-	int i;
-	int arg_max = var->fcn->cc ? rz_analysis_cc_max_arg(analysis, var->fcn->cc) : 0;
-	for (i = 0; i < arg_max; i++) {
+	int arg_max = 0;
+	if (RZ_STR_ISNOTEMPTY(var->fcn->cc)) {
+		arg_max = rz_analysis_cc_max_arg(analysis, var->fcn->cc);
+	}
+	for (int i = 0; i < arg_max; i++) {
 		const char *reg_arg = rz_analysis_cc_arg(analysis, var->fcn->cc, i);
 		if (reg_arg && !strcmp(reg->name, reg_arg)) {
 			return i;
@@ -783,7 +1019,6 @@ RZ_API bool rz_analysis_var_is_arg(RZ_NONNULL RzAnalysisVar *var) {
 	rz_return_val_if_fail(var, false);
 	switch (var->kind) {
 	case RZ_ANALYSIS_VAR_KIND_FORMAL_PARAMETER:
-	case RZ_ANALYSIS_VAR_KIND_UNSPECIFIED_PARAMETERS:
 		return true;
 	case RZ_ANALYSIS_VAR_KIND_VARIABLE:
 		return false;
@@ -797,6 +1032,7 @@ RZ_API bool rz_analysis_var_is_arg(RZ_NONNULL RzAnalysisVar *var) {
 		}
 		break;
 	}
+	default: break;
 	}
 
 	switch (var->storage.type) {
@@ -805,7 +1041,6 @@ RZ_API bool rz_analysis_var_is_arg(RZ_NONNULL RzAnalysisVar *var) {
 	case RZ_ANALYSIS_VAR_STORAGE_STACK:
 		return stack_offset_is_arg(var->fcn, var->storage.stack_off);
 	default:
-		rz_warn_if_reached();
 		return false;
 	}
 }
@@ -849,8 +1084,12 @@ static const char *get_regname(RzAnalysis *analysis, RzAnalysisValue *value) {
 	return name;
 }
 
+static inline bool is_not_read_nor_write(const RzAnalysisOpDirection direction) {
+	return direction != RZ_ANALYSIS_OP_DIR_READ && direction != RZ_ANALYSIS_OP_DIR_WRITE;
+}
+
 /**
- * Try to extract any args from a single op
+ * \brief Try to extract any args from a single op
  *
  * \param reg name of the register to look at for accesses
  * \param from_sp whether \p reg is the sp or bp
@@ -871,7 +1110,7 @@ static void extract_stack_var(RzAnalysis *analysis, RzAnalysisFunction *fcn, RzA
 		if ((delta < 0 && *sign == '+') || (delta >= 0 && *sign == '-')) {
 			continue;
 		}
-		if (!delta && op->direction != RZ_ANALYSIS_OP_DIR_READ && op->direction != RZ_ANALYSIS_OP_DIR_WRITE) {
+		if (!delta && is_not_read_nor_write(op->direction)) {
 			// avoid creating variables for just `mov rbp, rsp`, which would otherwise detect a var at rsp+0
 			// so for delta == 0, we only consider actual memory operations for now
 			continue;
@@ -936,6 +1175,11 @@ static void extract_stack_var(RzAnalysis *analysis, RzAnalysisFunction *fcn, RzA
 		if (*sign == '-') {
 			addend = -addend;
 		}
+		if (addend == 0 && is_not_read_nor_write(op->direction)) {
+			// avoid creating variables for just `mov rbp, rsp`, which would otherwise detect a var at rsp+0
+			// so for addend == 0, we only consider actual memory operations for now
+			goto beach;
+		}
 	}
 
 	if (!op->src[0] || !op->dst) {
@@ -1007,7 +1251,7 @@ static void extract_stack_var(RzAnalysis *analysis, RzAnalysisFunction *fcn, RzA
 		if (varname) {
 			RzAnalysisVarStorage stor;
 			rz_analysis_var_storage_init_stack(&stor, stack_off);
-			RzAnalysisVar *var = rz_analysis_function_set_var(fcn, &stor, vartype, analysis->bits / 8, varname);
+			RzAnalysisVar *var = rz_analysis_function_set_var(fcn, &stor, vartype, rz_analysis_guessed_mem_access_width(analysis), varname);
 			if (var) {
 				rz_analysis_var_set_access(var, reg, op->addr, rw, addend);
 			}
@@ -1023,7 +1267,7 @@ static void extract_stack_var(RzAnalysis *analysis, RzAnalysisFunction *fcn, RzA
 		}
 		char *varname = rz_str_newf("%s_%" PFMT64x "h", VARPREFIX, RZ_ABS(stor.stack_off));
 		if (varname) {
-			RzAnalysisVar *var = rz_analysis_function_set_var(fcn, &stor, NULL, analysis->bits / 8, varname);
+			RzAnalysisVar *var = rz_analysis_function_set_var(fcn, &stor, NULL, rz_analysis_guessed_mem_access_width(analysis), varname);
 			if (var) {
 				rz_analysis_var_set_access(var, reg, op->addr, rw, addend);
 			}
@@ -1354,60 +1598,85 @@ RZ_API RZ_OWN RzList /*<RzAnalysisVar *>*/ *rz_analysis_var_list(RZ_NONNULL RzAn
 	return list;
 }
 
-static int stackvar_comparator(const RzAnalysisVar *a, const RzAnalysisVar *b) {
-	if (!a || !b || a->storage.type != RZ_ANALYSIS_VAR_STORAGE_STACK || b->storage.type != RZ_ANALYSIS_VAR_STORAGE_STACK) {
-		return 0;
-	}
-	return a->storage.stack_off - b->storage.stack_off;
-}
-
 static int regvar_comparator(const RzAnalysisVar *a, const RzAnalysisVar *b) {
 	return (a && b) ? (a->argnum > b->argnum) - (a->argnum < b->argnum) : 0;
 }
 
+static int var_comparator(const RzAnalysisVar *a, const RzAnalysisVar *b, void *user) {
+	if (a->origin.kind == RZ_ANALYSIS_VAR_ORIGIN_DWARF && b->origin.kind == RZ_ANALYSIS_VAR_ORIGIN_DWARF) {
+		return (int)a->origin.dw_var->offset - b->origin.dw_var->offset;
+	}
+	if (a->origin.kind != b->origin.kind) {
+		return (int)a->origin.kind - b->origin.kind;
+	}
+	bool aa = rz_analysis_var_is_arg((RzAnalysisVar *)a);
+	bool bb = rz_analysis_var_is_arg((RzAnalysisVar *)a);
+	if ((aa && bb) || (!aa && !bb)) {
+		if (a->storage.type == RZ_ANALYSIS_VAR_STORAGE_REG && b->storage.type == RZ_ANALYSIS_VAR_STORAGE_REG) {
+			return regvar_comparator(a, b);
+		}
+		return rz_analysis_var_storage_cmp(&a->storage, &b->storage);
+	}
+	return aa ? -1 : 1;
+}
+
 /**
  * Populate \p cache with the variables from \p fcn and sort them according to their storage
  */
-RZ_API void rz_analysis_fcn_vars_cache_init(RzAnalysis *analysis, RzAnalysisFcnVarsCache *cache, RzAnalysisFunction *fcn) {
+RZ_API void rz_analysis_fcn_vars_cache_init(
+	RZ_NONNULL RZ_BORROW RzAnalysis *analysis,
+	RZ_NONNULL RZ_BORROW RZ_OUT RzAnalysisFcnVarsCache *cache,
+	RZ_NONNULL RZ_BORROW RzAnalysisFunction *fcn) {
 	rz_return_if_fail(analysis && cache && fcn);
-	cache->regvars = rz_analysis_var_list(fcn, RZ_ANALYSIS_VAR_STORAGE_REG);
-	cache->stackvars = rz_analysis_var_list(fcn, RZ_ANALYSIS_VAR_STORAGE_STACK);
-	rz_list_sort(cache->stackvars, (RzListComparator)stackvar_comparator);
+	cache->sorted_vars = rz_list_new_from_array((const void **)rz_pvector_data(&fcn->vars), rz_pvector_len(&fcn->vars));
+	cache->arg_vars = rz_list_new();
 	RzListIter *it;
 	RzAnalysisVar *var;
-	rz_list_foreach (cache->regvars, it, var) {
+	rz_list_foreach (cache->sorted_vars, it, var) {
 		var->argnum = rz_analysis_var_get_argnum(var);
+		if (rz_analysis_var_is_arg(var)) {
+			rz_list_append(cache->arg_vars, var);
+		}
 	}
-	rz_list_sort(cache->regvars, (RzListComparator)regvar_comparator);
+	rz_list_sort(cache->sorted_vars, (RzListComparator)var_comparator, NULL);
+	rz_list_sort(cache->arg_vars, (RzListComparator)var_comparator, NULL);
+}
+
+/**
+ * \brief Create variable cache with the variables from \p fcn and sort them according to their storage
+ * \param analysis The RzAnalysis instance
+ * \param fcn The RzAnalysis
+ * \return RzAnalysisFcnVarsCache or NULL
+ */
+RZ_API RZ_OWN RzAnalysisFcnVarsCache *rz_analysis_fcn_vars_cache_from_fcn(
+	RZ_NONNULL RZ_BORROW RzAnalysis *analysis,
+	RZ_NONNULL RZ_BORROW RzAnalysisFunction *fcn) {
+	rz_return_val_if_fail(analysis && fcn, NULL);
+	RzAnalysisFcnVarsCache *cache = RZ_NEW0(RzAnalysisFcnVarsCache);
+	if (!cache) {
+		return NULL;
+	}
+	rz_analysis_fcn_vars_cache_init(analysis, cache, fcn);
+	return cache;
 }
 
 RZ_API void rz_analysis_fcn_vars_cache_fini(RzAnalysisFcnVarsCache *cache) {
 	if (!cache) {
 		return;
 	}
-	rz_list_free(cache->regvars);
-	rz_list_free(cache->stackvars);
+	rz_list_free(cache->sorted_vars);
+	rz_list_free(cache->arg_vars);
 }
 
-static char *sig_from_debuginfo(RzAnalysis *analysis, char *fcn_name, const char *fcn_name_pre, const char *fcn_name_post) {
-	if (!rz_str_startswith(fcn_name, "dbg."))
+static char *sig_from_debuginfo(RzAnalysis *analysis, RZ_NONNULL RzAnalysisFunction *fcn, char *fcn_name, const char *fcn_name_pre, const char *fcn_name_post) {
+	if (!fcn->has_debuginfo || !rz_str_startswith(fcn_name, "dbg."))
 		return NULL;
 
-	const char *real_fcn_name = fcn_name + 4;
-	char *key = RZ_STR_ISNOTEMPTY(real_fcn_name) ? rz_str_newf("fcn.%s.sig", real_fcn_name) : NULL;
-	Sdb *sdb = sdb_ns(analysis->sdb, "dwarf", 0);
-	char *sig = sdb && key ? sdb_get(sdb, key, 0) : NULL;
-	free(key);
-	if (!sig)
+	RzCallable *callable = rz_type_func_get(analysis->typedb, fcn_name + 4);
+	if (!callable) {
 		return NULL;
-
-	char *highlighted_fcn_name = rz_str_newf("%s%s%s",
-		fcn_name_pre ? fcn_name_pre : "",
-		fcn_name,
-		fcn_name_post ? fcn_name_post : "");
-	char *highlighted_sig = rz_str_replace(sig, real_fcn_name, highlighted_fcn_name, false);
-	free(highlighted_fcn_name);
-	return highlighted_sig;
+	}
+	return rz_type_callable_as_string(analysis->typedb, callable);
 }
 
 /**
@@ -1428,7 +1697,7 @@ RZ_API char *rz_analysis_fcn_format_sig(RZ_NONNULL RzAnalysis *analysis, RZ_NONN
 		return NULL;
 	}
 
-	char *sig = sig_from_debuginfo(analysis, fcn_name, fcn_name_pre, fcn_name_post);
+	char *sig = sig_from_debuginfo(analysis, fcn, fcn_name, fcn_name_pre, fcn_name_post);
 	if (RZ_STR_ISNOTEMPTY(sig)) {
 		return sig;
 	}
@@ -1438,6 +1707,14 @@ RZ_API char *rz_analysis_fcn_format_sig(RZ_NONNULL RzAnalysis *analysis, RZ_NONN
 		return NULL;
 	}
 
+	RzAnalysisFcnVarsCache *cache = reuse_cache;
+	if (!cache) {
+		cache = rz_analysis_fcn_vars_cache_from_fcn(analysis, fcn);
+		if (!cache) {
+			return NULL;
+		}
+	}
+
 	char *type_fcn_name = rz_analysis_function_name_guess(analysis->typedb, fcn_name);
 	if (type_fcn_name && rz_type_func_exist(analysis->typedb, type_fcn_name)) {
 		RzType *ret_type = rz_type_func_ret(analysis->typedb, type_fcn_name);
@@ -1460,7 +1737,6 @@ RZ_API char *rz_analysis_fcn_format_sig(RZ_NONNULL RzAnalysis *analysis, RZ_NONN
 	}
 	rz_strbuf_append(buf, "(");
 
-	RzAnalysisFcnVarsCache *cache = reuse_cache;
 	if (type_fcn_name && rz_type_func_exist(analysis->typedb, type_fcn_name)) {
 		int i, argc = rz_type_func_args_count(analysis->typedb, type_fcn_name);
 		bool comma = true;
@@ -1485,68 +1761,25 @@ RZ_API char *rz_analysis_fcn_format_sig(RZ_NONNULL RzAnalysis *analysis, RZ_NONN
 	}
 	RZ_FREE(type_fcn_name);
 
-	if (!cache) {
-		cache = RZ_NEW0(RzAnalysisFcnVarsCache);
-		if (!cache) {
-			type_fcn_name = NULL;
-			goto beach;
-		}
-		rz_analysis_fcn_vars_cache_init(analysis, cache, fcn);
-	}
-
-	bool comma = true;
-	size_t tmp_len;
 	RzAnalysisVar *var;
 	RzListIter *iter;
-
-	rz_list_foreach (cache->regvars, iter, var) {
-		// assume self, error are always the last
-		if (!strcmp(var->name, "self") || !strcmp(var->name, "error")) {
-			rz_strbuf_slice(buf, 0, rz_strbuf_length(buf) - 2);
-			break;
-		}
+	rz_list_foreach (cache->arg_vars, iter, var) {
 		char *vartype = rz_type_as_string(analysis->typedb, var->type);
-		tmp_len = strlen(vartype);
+		size_t tmp_len = strlen(vartype);
 		rz_strbuf_appendf(buf, "%s%s%s%s", vartype,
 			tmp_len && vartype[tmp_len - 1] == '*' ? "" : " ",
-			var->name, iter->n ? ", " : "");
+			var->name, rz_list_iter_has_next(iter) ? ", " : "");
 		free(vartype);
 	}
 
-	RzList *args = rz_list_new();
-	rz_list_foreach (cache->stackvars, iter, var) {
-		if (!rz_analysis_var_is_arg(var)) {
-			continue;
-		}
-
-		if (var->kind == RZ_ANALYSIS_VAR_KIND_FORMAL_PARAMETER) {
-			rz_list_prepend(args, var);
-		} else {
-			rz_list_append(args, var);
-		}
-	}
-	rz_list_foreach (args, iter, var) {
-		if (!rz_list_empty(cache->regvars) && comma) {
-			rz_strbuf_append(buf, ", ");
-			comma = false;
-		}
-		char *vartype = rz_type_as_string(analysis->typedb, var->type);
-		tmp_len = strlen(vartype);
-		rz_strbuf_appendf(buf, "%s%s%s%s", vartype,
-			tmp_len && vartype[tmp_len - 1] == '*' ? "" : " ",
-			var->name, iter->n ? ", " : "");
-		free(vartype);
-	}
-	rz_list_free(args);
-
 beach:
-	rz_strbuf_append(buf, ");");
 	RZ_FREE(type_fcn_name);
 	if (!reuse_cache && cache) {
 		// !reuse_cache => we created our own cache
 		rz_analysis_fcn_vars_cache_fini(cache);
 		free(cache);
 	}
+	rz_strbuf_append(buf, ");");
 	return rz_strbuf_drain(buf);
 }
 
@@ -1596,3 +1829,17 @@ RZ_API void rz_analysis_fcn_vars_add_types(RzAnalysis *analysis, RZ_NONNULL RzAn
 	rz_type_func_save(analysis->typedb, callable);
 	rz_analysis_fcn_vars_cache_fini(&cache);
 }
+
+/**
+ * \brief Calculates the size of a variable
+ *
+ * Gets the type (RzType) of a variable and calculates its size
+ *
+ * \param analysis RzAnalysis instance
+ * \param var Variable to check
+ * \return The variable size in bits or `UT64_MAX` in case of failure
+ */
+RZ_API ut64 rz_analysis_var_size(const RzAnalysis *analysis, RZ_NONNULL RzAnalysisVar *var) {
+	rz_return_val_if_fail(analysis && var, UT64_MAX);
+	return rz_type_db_get_bitsize(analysis->typedb, var->type);
+}
