@@ -636,10 +636,7 @@ extern disassemble_info tm_print_insn_info;
  * bfd_getm32 - To retrieve the upper 16-bits of the ARCtangent-A5
  *              basecase (32-bit) instruction
  */
-static bfd_vma
-bfd_getm32(data)
-unsigned int data;
-{
+static bfd_vma bfd_getm32(unsigned int data) {
 	bfd_vma value = 0;
 
 	value = ((data & 0xff00) | (data & 0xff)) << 16;
@@ -651,10 +648,7 @@ unsigned int data;
  * bfd_getm32_ac - To retrieve the upper 8-bits of the ARCompact
  *                 16-bit instruction
  */
-static bfd_vma
-bfd_getm32_ac(data)
-unsigned int data;
-{
+static bfd_vma bfd_getm32_ac(unsigned int data) {
 	bfd_vma value = 0;
 
 	value = ((data & 0xff) << 8 | (data & 0xff00) >> 8);
