@@ -3,19 +3,12 @@
 
 #include <rz_asm.h>
 #include <rz_lib.h>
-#include <capstone/capstone.h>
 #include "../arch/ppc/libvle/vle.h"
 #include "../arch/ppc/libps/libps.h"
 
-static csh handle = 0;
+#include "cs_helper.h"
 
-static bool the_end(void *p) {
-	if (handle) {
-		cs_close(&handle);
-		handle = 0;
-	}
-	return true;
-}
+CAPSTONE_DEFINE_PLUGIN_FUNCTIONS(ppc);
 
 static int decompile_vle(RzAsm *a, RzAsmOp *op, const ut8 *buf, int len) {
 	vle_t *instr = 0;
@@ -54,10 +47,11 @@ static int decompile_ps(RzAsm *a, RzAsmOp *op, const ut8 *buf, int len) {
 	return op->size;
 }
 
-static int disassemble(RzAsm *a, RzAsmOp *op, const ut8 *buf, int len) {
-	static int omode = -1, obits = -1;
-	int n, ret, mode;
+static int ppc_disassemble(RzAsm *a, RzAsmOp *op, const ut8 *buf, int len) {
+	CapstoneContext *ctx = (CapstoneContext *)a->plugin_data;
+	int n, ret;
 	ut64 off = a->pc;
+	cs_mode mode = 0;
 	cs_insn *insn;
 	if (a->cpu && strncmp(a->cpu, "vle", 3) == 0) {
 		// vle is big-endian only
@@ -90,26 +84,29 @@ static int disassemble(RzAsm *a, RzAsmOp *op, const ut8 *buf, int len) {
 		break;
 	}
 	mode |= a->big_endian ? CS_MODE_BIG_ENDIAN : CS_MODE_LITTLE_ENDIAN;
-	if (mode != omode || a->bits != obits) {
-		cs_close(&handle);
-		handle = 0;
-		omode = mode;
-		obits = a->bits;
+	if (a->cpu && RZ_STR_EQ(a->cpu, "qpx")) {
+		mode |= CS_MODE_QPX;
 	}
-	if (handle == 0) {
-		ret = cs_open(CS_ARCH_PPC, mode, &handle);
-		if (ret != CS_ERR_OK) {
+
+	if (ctx->omode != mode) {
+		cs_close(&ctx->handle);
+		ctx->omode = -1;
+	}
+	if (!ctx->handle) {
+		ret = cs_open(CS_ARCH_PPC, mode, &ctx->handle);
+		if (ret) {
 			return -1;
 		}
+		ctx->omode = mode;
+		cs_option(ctx->handle, CS_OPT_DETAIL, CS_OPT_OFF);
 	}
+
 	op->size = 4;
-	cs_option(handle, CS_OPT_DETAIL, CS_OPT_OFF);
-	n = cs_disasm(handle, (const ut8 *)buf, len, off, 1, &insn);
+	n = cs_disasm(ctx->handle, (const ut8 *)buf, len, off, 1, &insn);
 	op->size = 4;
 	if (n > 0 && insn->size > 0) {
-		const char *opstr = sdb_fmt("%s%s%s", insn->mnemonic,
+		rz_asm_op_setf_asm(op, "%s%s%s", insn->mnemonic,
 			insn->op_str[0] ? " " : "", insn->op_str);
-		rz_asm_op_set_asm(op, opstr);
 		cs_free(insn, n);
 		return op->size;
 	}
@@ -125,11 +122,13 @@ RzAsmPlugin rz_asm_plugin_ppc_cs = {
 	.license = "BSD",
 	.author = "pancake",
 	.arch = "ppc",
-	.cpus = "ppc,vle,ps",
+	.cpus = "ppc,vle,ps,qpx",
 	.bits = 32 | 64,
 	.endian = RZ_SYS_ENDIAN_LITTLE | RZ_SYS_ENDIAN_BIG,
-	.fini = the_end,
-	.disassemble = &disassemble,
+	.init = ppc_init,
+	.fini = ppc_fini,
+	.disassemble = &ppc_disassemble,
+	.mnemonics = ppc_mnemonics,
 };
 
 #ifndef RZ_PLUGIN_INCORE
