@@ -134,7 +134,7 @@ static ut64 baddr(RzBinFile *bf) {
 	return 0x180000000;
 }
 
-void symbols_from_bin(RzDyldCache *cache, RzList /*<RzBinSymbol *>*/ *ret, RzBinFile *bf, RzDyldBinImage *bin, SetU *hash) {
+void symbols_from_bin(RzDyldCache *cache, RzPVector /*<RzBinSymbol *>*/ *ret, RzBinFile *bf, RzDyldBinImage *bin, SetU *hash) {
 	struct MACH0_(obj_t) *mach0 = bin_to_mach0(bf, bin);
 	if (!mach0) {
 		return;
@@ -166,7 +166,7 @@ void symbols_from_bin(RzDyldCache *cache, RzList /*<RzBinSymbol *>*/ *ret, RzBin
 		sym->ordinal = i;
 
 		set_u_add(hash, sym->vaddr);
-		rz_list_append(ret, sym);
+		rz_pvector_push(ret, sym);
 	}
 	MACH0_(mach0_free)
 	(mach0);
@@ -191,7 +191,7 @@ static bool __is_data_section(const char *name) {
 	return false;
 }
 
-static void sections_from_bin(RzList /*<RzBinSection *>*/ *ret, RzBinFile *bf, RzDyldBinImage *bin) {
+static void sections_from_bin(RzPVector /*<RzBinSection *>*/ *ret, RzBinFile *bf, RzDyldBinImage *bin) {
 	RzDyldCache *cache = (RzDyldCache *)bf->o->bin_obj;
 	if (!cache) {
 		return;
@@ -231,15 +231,15 @@ static void sections_from_bin(RzList /*<RzBinSection *>*/ *ret, RzBinFile *bf, R
 			ptr->vaddr = ptr->paddr;
 		}
 		ptr->perm = sections[i].perm;
-		rz_list_append(ret, ptr);
+		rz_pvector_push(ret, ptr);
 	}
 	free(sections);
 	MACH0_(mach0_free)
 	(mach0);
 }
 
-static RzList /*<RzBinVirtualFile *>*/ *virtual_files(RzBinFile *bf) {
-	RzList *ret = rz_list_newf((RzListFree)rz_bin_virtual_file_free);
+static RzPVector /*<RzBinVirtualFile *>*/ *virtual_files(RzBinFile *bf) {
+	RzPVector *ret = rz_pvector_new((RzPVectorFree)rz_bin_virtual_file_free);
 	if (!ret) {
 		return NULL;
 	}
@@ -252,7 +252,7 @@ static RzList /*<RzBinVirtualFile *>*/ *virtual_files(RzBinFile *bf) {
 		vf->buf = rz_dyldcache_new_rebasing_buf(cache);
 		vf->buf_owned = true;
 		vf->name = strdup(RZ_DYLDCACHE_VFILE_NAME_REBASED);
-		rz_list_push(ret, vf);
+		rz_pvector_push(ret, vf);
 	}
 	return ret;
 }
@@ -271,12 +271,12 @@ static int prot2perm(int x) {
 	return r;
 }
 
-static RzList /*<RzBinMap *>*/ *maps(RzBinFile *bf) {
+static RzPVector /*<RzBinMap *>*/ *maps(RzBinFile *bf) {
 	RzDyldCache *cache = (RzDyldCache *)bf->o->bin_obj;
 	if (!cache) {
 		return NULL;
 	}
-	RzList *ret = rz_list_newf((RzListFree)rz_bin_map_free);
+	RzPVector *ret = rz_pvector_new((RzPVectorFree)rz_bin_map_free);
 	if (!ret) {
 		return NULL;
 	}
@@ -284,7 +284,7 @@ static RzList /*<RzBinMap *>*/ *maps(RzBinFile *bf) {
 	for (ut32 i = 0; i < cache->n_maps; i++) {
 		RzBinMap *map = RZ_NEW0(RzBinMap);
 		if (!map) {
-			rz_list_free(ret);
+			rz_pvector_free(ret);
 			return NULL;
 		}
 		map->name = rz_str_newf("cache_map.%d", i);
@@ -296,17 +296,17 @@ static RzList /*<RzBinMap *>*/ *maps(RzBinFile *bf) {
 		if (rz_dyldcache_range_needs_rebasing(cache, map->paddr, map->psize)) {
 			map->vfile_name = strdup(RZ_DYLDCACHE_VFILE_NAME_REBASED);
 		}
-		rz_list_append(ret, map);
+		rz_pvector_push(ret, map);
 	}
 	return ret;
 }
 
-static RzList /*<RzBinSection *>*/ *sections(RzBinFile *bf) {
+static RzPVector /*<RzBinSection *>*/ *sections(RzBinFile *bf) {
 	RzDyldCache *cache = (RzDyldCache *)bf->o->bin_obj;
 	if (!cache) {
 		return NULL;
 	}
-	RzList *ret = rz_list_newf((RzListFree)rz_bin_section_free);
+	RzPVector *ret = rz_pvector_new((RzPVectorFree)rz_bin_section_free);
 	if (!ret) {
 		return NULL;
 	}
@@ -318,20 +318,22 @@ static RzList /*<RzBinSection *>*/ *sections(RzBinFile *bf) {
 	ut64 slide = rz_dyldcache_get_slide(cache);
 	if (slide) {
 		RzBinSection *section;
-		rz_list_foreach (ret, iter, section) {
+		void **it;
+		rz_pvector_foreach (ret, it) {
+			section = *it;
 			section->vaddr += slide;
 		}
 	}
 	return ret;
 }
 
-static RzList /*<RzBinSymbol *>*/ *symbols(RzBinFile *bf) {
+static RzPVector /*<RzBinSymbol *>*/ *symbols(RzBinFile *bf) {
 	RzDyldCache *cache = (RzDyldCache *)bf->o->bin_obj;
 	if (!cache) {
 		return NULL;
 	}
 
-	RzList *ret = rz_list_newf((RzListFree)rz_bin_symbol_free);
+	RzPVector *ret = rz_pvector_new((RzPVectorFree)rz_bin_symbol_free);
 	if (!ret) {
 		return NULL;
 	}
@@ -341,7 +343,7 @@ static RzList /*<RzBinSymbol *>*/ *symbols(RzBinFile *bf) {
 	rz_list_foreach (cache->bins, iter, bin) {
 		SetU *hash = set_u_new();
 		if (!hash) {
-			rz_list_free(ret);
+			rz_pvector_free(ret);
 			return NULL;
 		}
 		symbols_from_bin(cache, ret, bf, bin, hash);
@@ -352,7 +354,9 @@ static RzList /*<RzBinSymbol *>*/ *symbols(RzBinFile *bf) {
 	ut64 slide = rz_dyldcache_get_slide(cache);
 	if (slide) {
 		RzBinSymbol *sym;
-		rz_list_foreach (ret, iter, sym) {
+		void **it;
+		rz_pvector_foreach (ret, it) {
+			sym = *it;
 			sym->vaddr += slide;
 		}
 	}
@@ -365,13 +369,13 @@ static void destroy(RzBinFile *bf) {
 	rz_dyldcache_free(cache);
 }
 
-static RzList /*<RzBinClass *>*/ *classes(RzBinFile *bf) {
+static RzPVector /*<RzBinClass *>*/ *classes(RzBinFile *bf) {
 	RzDyldCache *cache = (RzDyldCache *)bf->o->bin_obj;
 	if (!cache) {
 		return NULL;
 	}
 
-	RzList *ret = rz_list_newf(free);
+	RzPVector *ret = rz_pvector_new(free);
 	if (!ret) {
 		return NULL;
 	}
@@ -477,7 +481,7 @@ static RzList /*<RzBinClass *>*/ *classes(RzBinFile *bf) {
 					}
 					num_of_unnamed_class++;
 				}
-				rz_list_append(ret, klass);
+				rz_pvector_push(ret, klass);
 			}
 
 			RZ_FREE(pointers);
@@ -491,7 +495,7 @@ static RzList /*<RzBinClass *>*/ *classes(RzBinFile *bf) {
 	return ret;
 
 beach:
-	rz_list_free(ret);
+	rz_pvector_free(ret);
 	rz_buf_free(owned_buf);
 	return NULL;
 }
