@@ -4,7 +4,6 @@
 #include <errno.h>
 #include <math.h> /* for ceill */
 #include <rz_util.h>
-#define RZ_NUM_USE_CALC 1
 
 static ut64 rz_num_tailff(RzNum *num, const char *hex);
 
@@ -218,26 +217,9 @@ RZ_API char *rz_num_units(char *buf, size_t len, ut64 num) {
 	return buf;
 }
 
-RZ_API const char *rz_num_get_name(RzNum *num, ut64 n) {
-	if (num->cb_from_value) {
-		int ok = 0;
-		const char *msg = num->cb_from_value(num, n, &ok);
-		if (msg && *msg) {
-			return msg;
-		}
-		if (ok) {
-			return msg;
-		}
-	}
-	return NULL;
-}
-
 static void error(RzNum *num, const char *err_str) {
 	if (num) {
 		num->nc.errors++;
-#if 0
-		num->nc.calc_err = err_str;
-#endif
 	}
 }
 
@@ -468,13 +450,6 @@ RZ_API ut64 rz_num_get(RZ_NULLABLE RzNum *num, RZ_NULLABLE const char *str) {
 			}
 			break;
 		default:
-#if 0
-			// sscanf (str, "%"PFMT64d"%n", &ret, &chars_read);
-// 32bit chop
-#if __WINDOWS__
-			ret = _strtoui64 (str, &endptr, 10);
-#endif
-#endif
 			errno = 0;
 			ret = strtoull(str, &endptr, 10);
 			if (errno == ERANGE) {
@@ -492,41 +467,6 @@ RZ_API ut64 rz_num_get(RZ_NULLABLE RzNum *num, RZ_NULLABLE const char *str) {
 	return ret;
 }
 
-#if !RZ_NUM_USE_CALC
-static ut64 rz_num_op(RzNum *num, char op, ut64 a, ut64 b) {
-	switch (op) {
-	case '+': return a + b;
-	case '-': return a - b;
-	case '*': return a * b;
-	case '/':
-		if (!b && num)
-			num->dbz = 1;
-		return b ? a / b : 0;
-	case '&': return a & b;
-	case '|': return a | b;
-	case '^': return a ^ b;
-	}
-	return b;
-}
-
-RZ_API static ut64 rz_num_math_internal(RzNum *num, char *s) {
-	ut64 ret = 0LL;
-	char *p = s;
-	int i, nop, op = 0;
-	for (i = 0; s[i]; i++) {
-		if (rz_num_is_op(s[i])) {
-			nop = s[i];
-			s[i] = '\0';
-			ret = rz_num_op(num, op, ret, rz_num_get(num, p));
-			op = s[i] = nop;
-			p = s + i + 1;
-			break;
-		}
-	}
-	return rz_num_op(op, ret, rz_num_get(num, p));
-}
-#endif /* !RZ_NUM_USE_CALC */
-
 /**
  * \brief Compute an numerical expression.
  *
@@ -535,7 +475,6 @@ RZ_API static ut64 rz_num_math_internal(RzNum *num, char *s) {
  * \return Evaluated expression's value.
  **/
 RZ_API ut64 rz_num_math(RzNum *num, const char *str) {
-#if RZ_NUM_USE_CALC
 	ut64 ret;
 	const char *err = NULL;
 	if (!str || !*str) {
@@ -553,72 +492,6 @@ RZ_API ut64 rz_num_math(RzNum *num, const char *str) {
 		num->value = ret;
 	}
 	return ret;
-#else
-	ut64 ret = 0LL;
-	char op = '+';
-	int len;
-	char *p, *s, *os;
-	char *group;
-	if (!str)
-		return 0LL;
-
-	len = strlen(str) + 1;
-	os = malloc(len + 1);
-
-	s = os;
-	memcpy(s, str, len);
-	for (; *s == ' '; s++)
-		;
-	p = s;
-
-	do {
-		group = strchr(p, '(');
-		if (group) {
-			group[0] = '\0';
-			ret = rz_num_op(op, ret, rz_num_math_internal(num, p));
-			for (; p < group; p += 1) {
-				if (rz_num_is_op(*p)) {
-					op = *p;
-					break;
-				}
-			}
-			group[0] = '(';
-			p = group + 1;
-			if (rz_str_delta(p, '(', ')') < 0) {
-				char *p2 = strchr(p, '(');
-				if (p2 != NULL) {
-					*p2 = '\0';
-					ret = rz_num_op(op, ret, rz_num_math_internal(num, p));
-					ret = rz_num_op(op, ret, rz_num_math(num, p2 + 1));
-					p = p2 + 1;
-					continue;
-				}
-				eprintf("something really bad has happened! can't find '('\n");
-			} else {
-				ret = rz_num_op(op, ret, rz_num_math_internal(num, p));
-			}
-		} else {
-			ret = rz_num_op(op, ret, rz_num_math_internal(num, p));
-		}
-	} while (0);
-
-	if (num) {
-		num->value = ret;
-	}
-	free(os);
-	return ret;
-#endif
-}
-
-/**
- * \brief Check if given string represents a float value or not.
- *
- * \param num RzNum instance.
- * \param str Numerical value.
- * \return Non zero value if float, 0 otherwise.
- **/
-RZ_API int rz_num_is_float(RzNum *num, const char *str) {
-	return (IS_DIGIT(*str) && (strchr(str, '.') || str[strlen(str) - 1] == 'f'));
 }
 
 RZ_API double rz_num_get_float(RzNum *num, const char *str) {
@@ -681,13 +554,6 @@ RZ_API int rz_num_to_trits(char *out, ut64 num) {
 	return true;
 }
 
-RZ_API ut64 rz_num_chs(int cylinder, int head, int sector, int sectorsize) {
-	if (sectorsize < 1) {
-		sectorsize = 512;
-	}
-	return (ut64)cylinder * (ut64)head * (ut64)sector * (ut64)sectorsize;
-}
-
 RZ_API int rz_num_conditional(RzNum *num, const char *str) {
 	char *lgt, *t, *p, *s = strdup(str);
 	int res = 0;
@@ -933,7 +799,7 @@ RZ_API int rz_num_between(RzNum *num, const char *input_value) {
 	return num->value = RZ_BETWEEN(ns[0], ns[1], ns[2]);
 }
 
-RZ_API bool rz_num_is_op(const char c) {
+static bool char_is_op(const char c) {
 	return c == '/' || c == '+' || c == '-' || c == '*' ||
 		c == '%' || c == '&' || c == '^' || c == '|';
 }
@@ -948,7 +814,7 @@ RZ_API int rz_num_str_len(const char *str) {
 	while (str[i] != '\0') {
 		switch (st) {
 		case 0: // number
-			while (!rz_num_is_op(str[i]) && str[i] != ' ' && str[i] != '\0') {
+			while (!char_is_op(str[i]) && str[i] != ' ' && str[i] != '\0') {
 				i++;
 				if (str[i] == '(') {
 					i += rz_num_str_len(str + i);
@@ -961,7 +827,7 @@ RZ_API int rz_num_str_len(const char *str) {
 			while (str[i] != '\0' && str[i] == ' ') {
 				i++;
 			}
-			if (!rz_num_is_op(str[i])) {
+			if (!char_is_op(str[i])) {
 				return len;
 			}
 			if (str[i] == ')') {
@@ -1009,14 +875,6 @@ RZ_API void *rz_num_dup(ut64 n) {
 	return (void *)hn;
 }
 
-RZ_API double rz_num_cos(double a) {
-	return cos(a);
-}
-
-RZ_API double rz_num_sin(double a) {
-	return sin(a);
-}
-
 /**
  * \brief Convert the base suffix to the numeric value
  */
