@@ -140,13 +140,13 @@ bool test_rz_io_pcache(void) {
 	io->ff = 1;
 	ut8 buf[8];
 	int fd = rz_io_fd_open(io, "malloc://3", RZ_PERM_RW, 0);
-	rz_io_map_add(io, fd, RZ_PERM_RW, 0LL, 0LL, 1); //8
+	rz_io_map_add(io, fd, RZ_PERM_RW, 0LL, 0LL, 1); // 8
 	rz_io_map_add(io, fd, RZ_PERM_RW, 1, 1, 1); //=
 	rz_io_map_add(io, fd, RZ_PERM_RW, 1, 2, 1); //=
 	rz_io_map_add(io, fd, RZ_PERM_RW, 1, 3, 1); //=
 	rz_io_map_add(io, fd, RZ_PERM_RW, 1, 4, 1); //=
 	rz_io_map_add(io, fd, RZ_PERM_RW, 1, 5, 1); //=
-	rz_io_map_add(io, fd, RZ_PERM_RW, 2, 6, 1); //D
+	rz_io_map_add(io, fd, RZ_PERM_RW, 2, 6, 1); // D
 	io->p_cache = 2;
 	io->va = true;
 	rz_io_fd_write_at(io, fd, 0, (const ut8 *)"8=D", 3);
@@ -349,6 +349,7 @@ bool test_rz_io_default(void) {
 	close(fd);
 	mu_assert_memeq(buf, (ut8 *)"FEDCBA0987654321", 0x10, "data has been correctly written at 0x50");
 	rz_file_rm(filename);
+	free(filename);
 	mu_end;
 }
 
@@ -475,7 +476,7 @@ bool test_rz_io_map_del(void) {
 
 	ut8 buf[4] = { 0x42, 0x42, 0x42, 0x42 };
 	bool red = rz_io_read_at_mapped(io, 0x300, buf, sizeof(buf));
-	mu_assert_true(red, "read before map");
+	mu_assert_false(red, "read before map");
 	mu_assert_memeq(buf, (const ut8 *)"\xff\xff\xff\xff", 4, "read before map contents");
 	FILL_DUMMY_IO
 	mu_assert_false(tracker.failed_unexpected, "unexpected del event");
@@ -492,11 +493,15 @@ bool test_rz_io_map_del(void) {
 	mu_assert_false(tracker.failed_unexpected, "unexpected del event");
 	mu_assert_false(rz_pvector_contains(maps, map01), "map unregistered");
 	red = rz_io_read_at_mapped(io, 0x300, buf, sizeof(buf));
-	mu_assert_true(red, "read after unmap");
+	mu_assert_false(red, "read after unmap");
 	mu_assert_memeq(buf, (const ut8 *)"\xff\xff\xff\xff", 4, "read after unmap");
 
+	rz_list_push(tracker.expect, map00);
+	rz_list_push(tracker.expect, map02);
+	rz_list_push(tracker.expect, map10);
+	rz_list_push(tracker.expect, map11);
+	rz_list_push(tracker.expect, map20);
 	rz_io_free(io);
-	// free should not emit any events, we just know everything is closed
 	mu_assert_eq(rz_list_length(tracker.expect), 0, "missing del event");
 	mu_assert_false(tracker.failed_unexpected, "unexpected del event");
 	rz_list_free(tracker.expect);
@@ -531,11 +536,13 @@ bool test_rz_io_map_del_for_fd(void) {
 	mu_assert_false(rz_pvector_contains(maps, map02), "map unregistered");
 	ut8 buf[4] = { 0x42, 0x42, 0x42, 0x42 };
 	bool red = rz_io_read_at_mapped(io, 0x300, buf, sizeof(buf));
-	mu_assert_true(red, "read after unmap");
+	mu_assert_false(red, "read after unmap");
 	mu_assert_memeq(buf, (const ut8 *)"\xff\xff\xff\xff", 4, "read after unmap");
 
+	rz_list_push(tracker.expect, map10);
+	rz_list_push(tracker.expect, map11);
+	rz_list_push(tracker.expect, map20);
 	rz_io_free(io);
-	// free should not emit any events, we just know everything is closed
 	mu_assert_eq(rz_list_length(tracker.expect), 0, "missing del event");
 	mu_assert_false(tracker.failed_unexpected, "unexpected del event");
 	rz_list_free(tracker.expect);
@@ -570,11 +577,13 @@ bool test_rz_io_map_del_on_close(void) {
 	mu_assert_false(rz_pvector_contains(maps, map02), "map unregistered");
 	ut8 buf[4] = { 0x42, 0x42, 0x42, 0x42 };
 	bool red = rz_io_read_at_mapped(io, 0x300, buf, sizeof(buf));
-	mu_assert_true(red, "read after unmap");
+	mu_assert_false(red, "read after unmap");
 	mu_assert_memeq(buf, (const ut8 *)"\xff\xff\xff\xff", 4, "read after unmap");
 
+	rz_list_push(tracker.expect, map10);
+	rz_list_push(tracker.expect, map11);
+	rz_list_push(tracker.expect, map20);
 	rz_io_free(io);
-	// free should not emit any events, we just know everything is closed
 	mu_assert_eq(rz_list_length(tracker.expect), 0, "missing del event");
 	mu_assert_false(tracker.failed_unexpected, "unexpected del event");
 	rz_list_free(tracker.expect);
@@ -610,7 +619,7 @@ bool test_rz_io_map_del_on_close_all(void) {
 	mu_assert_true(rz_pvector_empty(maps), "map unregistered");
 	ut8 buf[4] = { 0x42, 0x42, 0x42, 0x42 };
 	bool red = rz_io_read_at_mapped(io, 0x300, buf, sizeof(buf));
-	mu_assert_true(red, "read after unmap");
+	mu_assert_false(red, "read after unmap");
 	mu_assert_memeq(buf, (const ut8 *)"\xff\xff\xff\xff", 4, "read after unmap");
 
 	rz_io_free(io);
